/*
 *	file:		code03e8.c
 *	project:	SilkClockHack
 *	content:	main functions
 *	updated:	20.08.2001
 *
 * copyright: Collin R. Mulliner <palm@mulliner.org>
 * web: www.mulliner.org/palm/
 *
 */

/*
 *  This file is part of SilkClockHack.
 *
 *  SilkClockHack is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SilkClockHack is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SilkClockHack; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sk.h"
#include "callback.h"

int do_work();
void MyEvtGetSysEvent(SysEventType *event, Int32 timeout);

		
void MyEvtGetSysEvent(SysEventType *event, Int32 timeout)
{
	void (*old_evtgetevent)(SysEventType*, Int32);
	UInt32 old_trap;
	UInt32 version;
	
	
	CALLBACK_PROLOGUE
	
	FtrGet(CRID, trapIDEvtGetSysEvent, &old_trap);
	old_evtgetevent = (void (*)(SysEventType*, Int32)) old_trap;

	if (_TRGSilkFeaturePresent(&version)) do_work();
			
	old_evtgetevent(event, timeout);
		
	CALLBACK_EPILOGUE
}

/*
 *	do the main work
 */
int do_work()
{
	char date_str[50];
	char time_str[50];
	char batt_str[50];
	char mem_str[50];
	Boolean charging;
	UInt8 percent;
	UInt16 volts;
	UInt32 freeMem, totalMem, max, freeB;
	UInt16 heapID;
	Int16 nCards, i, i2;
	UInt32 cur_secs;
	DateTimeType cur_datetime;
	WinHandle old_win, silk_win;
	RGBColorType black = {0,0,0,0};
	UInt32 cfgData;
	UInt8 *cfgDataP;
	UInt16 dataSize;
	VgaScreenModeType sMode;
	VgaRotateModeType rMode;
	UInt16 pos1, pos2;
	UInt32 ctl_status;
	Boolean winSize;

	
	CALLBACK_PROLOGUE

	if (FtrGet(CRID, SILK_CLOCK_HACK_CTL_FTR, &ctl_status) != 0) {
		ctl_status = SILK_CLOCK_HACK_SHOW;
	}
	
	winSize = SilkWindowMaximized();	// is the silk window max or min
	
	if (winSize && (ctl_status & SILK_CLOCK_HACK_DONT_SHOW_MAX) == SILK_CLOCK_HACK_DONT_SHOW_MAX) {
		CALLBACK_EPILOGUE
		return(0);
	}
	else if (!winSize && (ctl_status & SILK_CLOCK_HACK_DONT_SHOW_MIN) == SILK_CLOCK_HACK_DONT_SHOW_MIN) {
		CALLBACK_EPILOGUE
		return(0);
	}
	
	// check MurrayD's Keyboard
	if (winSize && FtrGet(MD_KB_CRID, MD_KB_FTR_NUM, &ctl_status) == 0) {	// keyboard is active !
		// don't draw clock !
		CALLBACK_EPILOGUE
		return(0);
	}
		
	// GET - cfg
	cfgDataP = (UInt8*) &cfgData;
	
	if (FtrGet(CRID, CFG_FTR, &cfgData) != 0) {
		if (PrefGetAppPreferences(CRID, 0, &cfgData, &dataSize, true) == noPreferenceFound) {
			// default prefs
			*(cfgDataP) = 0 | TIME_MAX_ON | TIME_MIN_ON;
			*(cfgDataP+1) = 0 | DATE_MAX_ON | DATE_MIN_ON;
			*(cfgDataP+2) = 0 | BAT_OFF;
			*(cfgDataP+3) = 0 | MEM_OFF;
		}
		else {
			FtrSet(CRID, CFG_FTR, cfgData);
		}
	}
	// ENDE > GET - cfg
		
	// get screen and rotation mode 
	VgaGetScreenMode(&sMode, &rMode);
	
	if (rMode == rotateModeNone || rMode == rotateMode180) {
		pos1 = 162;
		pos2 = 92;
	}
	else if (rMode == rotateMode90 || rMode == rotateMode270) {
		pos1 = 162+40;
		pos2 = 92+40;
	}

	// save draw mode	
	WinPushDrawState();
	// set new draw mode
	WinSetDrawMode(winPaint);
	// get the handle for the silk screen
	silk_win = SilkGetWindow();
	// make the silk screen the current draw screen
	old_win = WinSetDrawWindow(silk_win);
	// set color to black
	WinSetTextColor(WinRGBToIndex(&black));
	// set the right font
	FntSetFont(0);
	
	if (*(cfgDataP) != 0 || *(cfgDataP+1) != 0) {
		
		cur_secs = TimGetSeconds();
		TimSecondsToDateTime(cur_secs, &cur_datetime);

		StrCopy(date_str, " ");
		StrCopy(time_str, " ");

		DateToAscii(cur_datetime.month, cur_datetime.day, cur_datetime.year, PrefGetPreference(prefLongDateFormat), date_str+1);
		TimeToAscii(cur_datetime.hour, cur_datetime.minute, PrefGetPreference(prefTimeFormat), time_str+1);

		StrCat(date_str, " ");
		StrCat(time_str, " ");
/*
		WinPushDrawState();
			
		WinSetDrawMode(winPaint);
		
		silk_win = SilkGetWindow();
			
		old_win = WinSetDrawWindow(silk_win);

		// set color to black
		WinSetTextColor(WinRGBToIndex(&black));
		// set the right font
		FntSetFont(0);
*/
		if (winSize) {
			if ((*(cfgDataP) & TIME_MAX_ON)) WinPaintChars(time_str, StrLen(time_str), pos1-(FntLineWidth(time_str, StrLen(time_str))/2), 2);
			if ((*(cfgDataP+1) & DATE_MAX_ON)) WinPaintChars(date_str, StrLen(date_str), pos2-(FntLineWidth(date_str, StrLen(date_str))/2), 2);
		}
		else {
			if ((*(cfgDataP) & TIME_MIN_ON)) WinPaintChars(time_str, StrLen(time_str), pos1-(FntLineWidth(time_str, StrLen(time_str))/2), 4);
			if ((*(cfgDataP+1) & DATE_MIN_ON)) WinPaintChars(date_str, StrLen(date_str), pos2-(FntLineWidth(date_str, StrLen(date_str))/2), 4);
		}
/*		WinSetDrawWindow(old_win);
	
		WinPopDrawState(); */
	}
	
	if (winSize && (*(cfgDataP+2) != BAT_OFF || *(cfgDataP+3) != MEM_OFF)) {
		
		volts = SysBatteryInfo(false, NULL, NULL, NULL, NULL, &charging, &percent);
				
		if (*(cfgDataP+3) != MEM_OFF) {	// lots of work !
			totalMem = 0;
			freeMem = 0;
			nCards = MemNumCards();
			for (i = 0; i < nCards; i++) {
				for (i2 = 0; i2 < MemNumRAMHeaps(i); i2++) {
					heapID = MemHeapID(i, i2);
					if (!MemHeapDynamic(heapID)) {
						totalMem += MemHeapSize(heapID);
						MemHeapFreeBytes(heapID, &freeB, &max);
						freeMem += freeB;
					}
				}
			}
		
		//	MemCardInfo(0, NULL, NULL, NULL, NULL, NULL, &totalMem, &freeMem);
		}
		
		// battery
		if (charging) {
			StrCopy(batt_str, " charging ");
		}
		else if (*(cfgDataP+2) & BAT_PERC_ON) {
			StrCopy(batt_str, " B ");
			StrIToA(batt_str+StrLen(batt_str), percent);
			StrCat(batt_str, "% ");
		}
		else if (*(cfgDataP+2) & BAT_VOLT_ON) {
			StrCopy(batt_str, " ");
			StrIToA(batt_str+StrLen(batt_str), volts/100);
			StrCat(batt_str, ".");
			if ((volts%100) < 10) StrCat(batt_str, "0");
			StrIToA(batt_str+StrLen(batt_str), volts%100);
			StrCat(batt_str, "v ");
		}
		// memory
		if (*(cfgDataP+3) & MEM_PERC_ON) {
			StrCopy(mem_str, " M ");
			StrIToA(mem_str+StrLen(mem_str), freeMem / (totalMem/100));
			StrCat(mem_str, "% ");
		}
		else if (*(cfgDataP+3) & MEM_KB_ON) {
			StrCopy(mem_str, " ");
			StrIToA(mem_str+StrLen(mem_str), freeMem/1024);
			StrCat(mem_str, "K ");
		}
		
/*		WinPushDrawState();
			
		silk_win = SilkGetWindow();
		
		WinSetDrawMode(winPaint);
			
		old_win = WinSetDrawWindow(silk_win);

		// set color to black
		WinSetTextColor(WinRGBToIndex(&black));
		// set the right font
		FntSetFont(0);
*/
		if (*(cfgDataP+2) != BAT_OFF) WinPaintChars(batt_str, StrLen(batt_str), pos1-(FntLineWidth(batt_str, StrLen(batt_str))/2), 70-((charging) ? 1 : 0));
		if (*(cfgDataP+3) != MEM_OFF) WinPaintChars(mem_str, StrLen(mem_str), pos2-(FntLineWidth(mem_str, StrLen(mem_str))/2), 70);
/*
		WinSetDrawWindow(old_win);
	
		WinPopDrawState();
	*/	
	}

	// reset the old draw window
	WinSetDrawWindow(old_win);
	// reset the old draw mode
	WinPopDrawState();
	
	CALLBACK_EPILOGUE
			
	return(1);
}
