/*
 *	file:		code07d0.c
 *	project:	SilkClockHack
 *	content:	config
 *	updated:	11.05.2001
 *
 * copyright: Collin R. Mulliner <palm@mulliner.org>
 * web: www.mulliner.org/palm/
 *
 */

/*
 *  This file is part of SilkClockHack.
 *
 *  SilkClockHack is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SilkClockHack is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SilkClockHack; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sk.h"
#include "callback.h"

// protos
static Boolean cfg_Handler(EventPtr event);
void getSetPrefs(Boolean set, UInt8 *tCfg, UInt8 *dCfg, UInt8 *bCfg, UInt8 *mCfg);
void setFtr(UInt8 *tCfg, UInt8 *dCfg, UInt8 *bCfg, UInt8 *mCfg);

static Boolean cfg_Handler(EventPtr event)
{
	Boolean handled = false;
	FormPtr form;
	UInt8 cfg[4];

	
	#ifdef CB_ALL
	CALLBACK_PROLOGUE
	#endif CB_ALL

	form = FrmGetActiveForm();
			
	switch (event->eType) {

	case    frmOpenEvent:
		// load prefs
		getSetPrefs(false, &cfg[0], &cfg[1], &cfg[2], &cfg[3]);
		// set feature
		setFtr(&cfg[0], &cfg[1], &cfg[2], &cfg[3]);				
		
		// set push buttons
		FrmSetControlGroupSelection(form, time_grp, timeOff_cfg_button - (cfg[0] & TIME_MAX_ON));
		FrmSetControlGroupSelection(form, time_min_grp, timeOff_min_cfg_button - ((cfg[0] & TIME_MIN_ON)>>1));
		
		FrmSetControlGroupSelection(form, date_grp, dateOff_cfg_button - (cfg[1] & DATE_MAX_ON));
		FrmSetControlGroupSelection(form, date_min_grp, dateOff_min_cfg_button - ((cfg[1] & DATE_MIN_ON)>>1));
		
		FrmSetControlGroupSelection(form, battery_grp, batOff_cfg_button - cfg[2]);
		FrmSetControlGroupSelection(form, memory_grp, memOff_cfg_button - cfg[3]);
				
		FrmDrawForm(form);
		break;

	case	ctlSelectEvent:
		switch (event->data.ctlSelect.controlID) {
		case	cancel_cfg_button:
			FrmGotoForm(9000);	// back to HackMaster
			handled = true;
			break;
			
		case	ok_cfg_button:
			
			cfg[0] = 0 | (timeOff_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, time_grp))) | 
					((timeOff_min_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, time_min_grp)))<<1);
			
			cfg[1] = 0 | (dateOff_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, date_grp))) |
					((dateOff_min_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, date_min_grp)))<<1);
			
			cfg[2] = batOff_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, battery_grp));
			
			cfg[3] = memOff_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, memory_grp));
				
			// save prefs
			getSetPrefs(true, &cfg[0], &cfg[1], &cfg[2], &cfg[3]);
			
			FrmGotoForm(9000);	// back to HackMaster
			handled = true;
			break;
			
		case	timeOn_min_cfg_button:
		case	timeOff_min_cfg_button:
		case	timeOn_cfg_button:
		case	timeOff_cfg_button:
			
			cfg[0] = 0 | (timeOff_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, time_grp))) | 
					((timeOff_min_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, time_min_grp)))<<1);
						
			setFtr(&cfg[0], NULL, NULL, NULL);
			break;

		case	dateOn_min_cfg_button:
		case	dateOff_min_cfg_button:			
		case	dateOn_cfg_button:
		case	dateOff_cfg_button:
			
			cfg[1] = 0 | (dateOff_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, date_grp))) |
					((dateOff_min_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, date_min_grp)))<<1);
						
			setFtr(NULL, &cfg[1], NULL, NULL);
			break;
			
		case	batVolts_cfg_button:
		case	batPerc_cfg_button:
		case	batOff_cfg_button:
			
			cfg[2] = batOff_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, battery_grp));
			
			setFtr(NULL, NULL, &cfg[2], NULL);
			break;
			
		case	memKb_cfg_button:
		case	memPerc_cfg_button:
		case	memOff_cfg_button:
			
			cfg[3] = memOff_cfg_button - FrmGetObjectId(form, FrmGetControlGroupSelection(form, memory_grp));
			
			setFtr(NULL, NULL, NULL, &cfg[3]);
			break;
		}
		break;
		
	case	menuEvent:
		switch (event->data.menu.itemID) {
		}
		break;

        case	nilEvent:
		break;
	}

	#ifdef CB_ALL
	CALLBACK_EPILOGUE
	#endif CB_ALL

	return(handled);
}

/*
 *	get and set the prefs
 */
void getSetPrefs(Boolean set, UInt8 *tCfg, UInt8 *dCfg, UInt8 *bCfg, UInt8 *mCfg)
{
	UInt8 cfgData[4];
	UInt16 dataSize = 4;
				
	
	if (set) {
		cfgData[0] = *(tCfg);
		cfgData[1] = *(dCfg);
		cfgData[2] = *(bCfg);
		cfgData[3] = *(mCfg);
		
		PrefSetAppPreferences(CRID, 0, 0, &cfgData, 4, true);
	}
	else {	// get prefs
		if (PrefGetAppPreferences(CRID, 0, &cfgData, &dataSize, true) == noPreferenceFound) {
			// default prefs
			*(tCfg) = 0 | TIME_MAX_ON | TIME_MIN_ON;
			*(dCfg) = 0 | DATE_MAX_ON | DATE_MIN_ON;
			*(bCfg) = 0 | BAT_OFF;
			*(mCfg) = 0 | MEM_OFF;
		}
		else {
			// loaded prefs
			*(tCfg) = cfgData[0];
			*(dCfg) = cfgData[1];
			*(bCfg) = cfgData[2];
			*(mCfg) = cfgData[3];
		}
	}
}

/*
 *	set feature
 */
void setFtr(UInt8 *tCfg, UInt8 *dCfg, UInt8 *bCfg, UInt8 *mCfg)
{
	UInt32 cfgData;
	UInt8 *cfgDataP;
	

	if (FtrGet(CRID, CFG_FTR, &cfgData) != 0) {
		MemSet(&cfgData, 4, 0);
	}
	
	cfgDataP = (UInt8*) &cfgData;
	
	if (tCfg != NULL) *(cfgDataP) = *(tCfg);
	if (dCfg != NULL) *(cfgDataP+1) = *(dCfg);
	if (bCfg != NULL) *(cfgDataP+2) = *(bCfg);
	if (mCfg != NULL) *(cfgDataP+3) = *(mCfg);
	
	FtrSet(CRID, CFG_FTR, cfgData);
}
